@@ -70,6 +70,20 @@ void array_reset(array *a) {
 	a->unique_ndx = 0;
 }
 
+void array_reset_data_strings(array *a) {
+	if (!a) return;
+
+	for (size_t i = 0; i < a->used; ++i) {
+		data_string * const ds = (data_string *)a->data[i];
+		/*force_assert(ds->type == TYPE_STRING);*/
+		buffer_reset(ds->key);
+		buffer_reset(ds->value);
+	}
+
+	a->used = 0;
+	a->unique_ndx = 0;
+}
+
 data_unset *array_pop(array *a) {
 	data_unset *du;
 
@@ -83,6 +97,10 @@ data_unset *array_pop(array *a) {
 	return du;
 }
 
+static int array_keycmp(const char *a, size_t alen, const char *b, size_t blen) {
+    return alen < blen ? -1 : alen > blen ? 1 : buffer_caseless_compare(a, alen, b, blen);
+}
+
 /* returns index of element or ARRAY_NOT_FOUND
  * if rndx != NULL it stores the position in a->sorted[] where the key needs
  * to be inserted
@@ -97,9 +115,8 @@ static size_t array_get_index(const array *a, const char *key, size_t keylen, si
 
 	while (lower != upper) {
 		size_t probe = (lower + upper) / 2;
-		int cmp = buffer_caseless_compare(key, keylen, CONST_BUF_LEN(a->data[a->sorted[probe]]->key));
-		assert(lower < upper); /* from loop invariant (lower <= upper) + (lower != upper) */
-		assert((lower <= probe) && (probe < upper)); /* follows from lower < upper */
+		const buffer *b = a->data[a->sorted[probe]]->key;
+		int cmp = array_keycmp(key, keylen, CONST_BUF_LEN(b));
 
 		if (cmp == 0) {
 			/* found */
