@@ -65,13 +65,20 @@ void buffer_commit(buffer *b, size_t size);
  */
 void buffer_string_set_length(buffer *b, size_t len);
 
+/* clear buffer
+ * - invalidate buffer contents
+ * - unsets used chars but does not modify existing ptr contents
+ *   (b->ptr *is not* set to an empty, '\0'-terminated string "")
+ */
+static inline void buffer_clear(buffer *b);
+
 void buffer_copy_string(buffer *b, const char *s);
 void buffer_copy_string_len(buffer *b, const char *s, size_t s_len);
-void buffer_copy_buffer(buffer *b, const buffer *src);
+static inline void buffer_copy_buffer(buffer *b, const buffer *src);
 
 void buffer_append_string(buffer *b, const char *s);
 void buffer_append_string_len(buffer *b, const char *s, size_t s_len);
-void buffer_append_string_buffer(buffer *b, const buffer *src);
+static inline void buffer_append_string_buffer(buffer *b, const buffer *src);
 
 #define buffer_append_uint_hex(b,len) buffer_append_uint_hex_lc((b),(len))
 void buffer_append_uint_hex_lc(buffer *b, uintmax_t len);
@@ -127,6 +134,7 @@ void buffer_copy_string_encoded_cgi_varnames(buffer *b, const char *s, size_t s_
 
 void buffer_urldecode_path(buffer *url);
 void buffer_urldecode_query(buffer *url);
+int buffer_is_valid_UTF8(const buffer *b);
 void buffer_path_simplify(buffer *dest, buffer *src);
 
 void buffer_to_lower(buffer *b);
@@ -162,6 +170,7 @@ static inline int light_isalnum(int c) {
 static inline size_t buffer_string_length(const buffer *b); /* buffer string length without terminating 0 */
 static inline size_t buffer_string_space(const buffer *b); /* maximum length of string that can be stored without reallocating */
 static inline void buffer_append_slash(buffer *b); /* append '/' no non-empty strings not ending in '/' */
+void buffer_append_path_len(buffer *b, const char *a, size_t alen); /* join strings with '/', if '/' not present */
 
 #define BUFFER_APPEND_STRING_CONST(x, y) \
 	buffer_append_string_len(x, y, sizeof(y) - 1)
@@ -194,9 +203,15 @@ static inline size_t buffer_string_length(const buffer *b) {
 }
 
 static inline size_t buffer_string_space(const buffer *b) {
-	if (NULL == b || b->size == 0) return 0;
-	if (0 == b->used) return b->size - 1;
-	return b->size - b->used;
+	return NULL != b && b->size ? b->size - (b->used | (0 == b->used)) : 0;
+}
+
+static inline void buffer_copy_buffer(buffer *b, const buffer *src) {
+	buffer_copy_string_len(b, CONST_BUF_LEN(src));
+}
+
+static inline void buffer_append_string_buffer(buffer *b, const buffer *src) {
+	buffer_append_string_len(b, CONST_BUF_LEN(src));
 }
 
 static inline void buffer_append_slash(buffer *b) {
@@ -204,4 +219,8 @@ static inline void buffer_append_slash(buffer *b) {
 	if (len > 0 && '/' != b->ptr[len-1]) BUFFER_APPEND_STRING_CONST(b, "/");
 }
 
+static inline void buffer_clear(buffer *b) {
+	b->used = 0;
+}
+
 #endif
