@@ -57,7 +57,7 @@ static data_unset *configparser_get_variable(config_t *ctx, const buffer *key) {
 #endif
     if (NULL != (du = array_get_element_klen(dc->value, CONST_BUF_LEN(key)))) {
       du = du->fn->copy(du);
-      buffer_reset(du->key);
+      buffer_clear(du->key);
       return du;
     }
   }
@@ -1056,15 +1056,17 @@ static void yy_reduce(
       case 16:
 #line 319 "../../src/configparser.y"
 {
+  buffer *b;
   yygotominor.yy91 = (data_unset *)data_string_init();
-  buffer_move(((data_string *)(yygotominor.yy91))->value, yymsp[0].minor.yy0);
-  buffer_free(yymsp[0].minor.yy0);
+  b = ((data_string *)(yygotominor.yy91))->value;
+  buffer_free(b);
+  ((data_string *)(yygotominor.yy91))->value = yymsp[0].minor.yy0;
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1064 "./configparser.c"
+#line 1066 "./configparser.c"
         break;
       case 17:
-#line 326 "../../src/configparser.y"
+#line 328 "../../src/configparser.y"
 {
   char *endptr;
   yygotominor.yy91 = (data_unset *)data_integer_init();
@@ -1079,39 +1081,39 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1082 "./configparser.c"
+#line 1084 "./configparser.c"
         break;
       case 18:
-#line 340 "../../src/configparser.y"
+#line 342 "../../src/configparser.y"
 {
   yygotominor.yy91 = (data_unset *)data_array_init();
   array_free(((data_array *)(yygotominor.yy91))->value);
   ((data_array *)(yygotominor.yy91))->value = yymsp[0].minor.yy42;
   yymsp[0].minor.yy42 = NULL;
 }
-#line 1092 "./configparser.c"
+#line 1094 "./configparser.c"
         break;
       case 19:
-#line 346 "../../src/configparser.y"
+#line 348 "../../src/configparser.y"
 {
   yygotominor.yy42 = array_init();
 }
-#line 1099 "./configparser.c"
+#line 1101 "./configparser.c"
   yy_destructor(9,&yymsp[-1].minor);
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 20:
-#line 349 "../../src/configparser.y"
+#line 351 "../../src/configparser.y"
 {
   yygotominor.yy42 = yymsp[-1].minor.yy42;
   yymsp[-1].minor.yy42 = NULL;
 }
-#line 1109 "./configparser.c"
+#line 1111 "./configparser.c"
   yy_destructor(9,&yymsp[-2].minor);
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 21:
-#line 354 "../../src/configparser.y"
+#line 356 "../../src/configparser.y"
 {
   yygotominor.yy42 = NULL;
   if (ctx->ok) {
@@ -1133,20 +1135,20 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1136 "./configparser.c"
+#line 1138 "./configparser.c"
   yy_destructor(11,&yymsp[-1].minor);
         break;
       case 22:
-#line 376 "../../src/configparser.y"
+#line 378 "../../src/configparser.y"
 {
   yygotominor.yy42 = yymsp[-1].minor.yy42;
   yymsp[-1].minor.yy42 = NULL;
 }
-#line 1145 "./configparser.c"
+#line 1147 "./configparser.c"
   yy_destructor(11,&yymsp[0].minor);
         break;
       case 23:
-#line 381 "../../src/configparser.y"
+#line 383 "../../src/configparser.y"
 {
   yygotominor.yy42 = NULL;
   if (ctx->ok) {
@@ -1157,18 +1159,18 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1160 "./configparser.c"
+#line 1162 "./configparser.c"
         break;
       case 24:
-#line 392 "../../src/configparser.y"
+#line 394 "../../src/configparser.y"
 {
   yygotominor.yy91 = yymsp[0].minor.yy91;
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1168 "./configparser.c"
+#line 1170 "./configparser.c"
         break;
       case 25:
-#line 396 "../../src/configparser.y"
+#line 398 "../../src/configparser.y"
 {
   yygotominor.yy91 = NULL;
   if (ctx->ok) {
@@ -1182,7 +1184,7 @@ static void yy_reduce(
   buffer_free(yymsp[-2].minor.yy29);
   yymsp[-2].minor.yy29 = NULL;
 }
-#line 1185 "./configparser.c"
+#line 1187 "./configparser.c"
   yy_destructor(12,&yymsp[-1].minor);
         break;
       case 26:
@@ -1191,31 +1193,31 @@ static void yy_reduce(
       case 27:
         break;
       case 28:
-#line 413 "../../src/configparser.y"
+#line 415 "../../src/configparser.y"
 {
   data_config *dc;
   dc = (data_config *)array_get_element_klen(ctx->srv->config_context, CONST_STR_LEN("global"));
   force_assert(dc);
   configparser_push(ctx, dc, 0);
 }
-#line 1201 "./configparser.c"
+#line 1203 "./configparser.c"
   yy_destructor(13,&yymsp[0].minor);
         break;
       case 29:
-#line 420 "../../src/configparser.y"
+#line 422 "../../src/configparser.y"
 {
   force_assert(ctx->current);
   configparser_pop(ctx);
   force_assert(ctx->current);
 }
-#line 1211 "./configparser.c"
+#line 1213 "./configparser.c"
         /* No destructor defined for globalstart */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 30:
-#line 426 "../../src/configparser.y"
+#line 428 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1234,12 +1236,12 @@ static void yy_reduce(
   yymsp[-3].minor.yy18 = NULL;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1237 "./configparser.c"
+#line 1239 "./configparser.c"
         /* No destructor defined for eols */
   yy_destructor(16,&yymsp[-1].minor);
         break;
       case 31:
-#line 445 "../../src/configparser.y"
+#line 447 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1300,20 +1302,20 @@ static void yy_reduce(
   yymsp[-3].minor.yy18 = NULL;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1303 "./configparser.c"
+#line 1305 "./configparser.c"
         /* No destructor defined for eols */
   yy_destructor(16,&yymsp[-1].minor);
         break;
       case 32:
-#line 506 "../../src/configparser.y"
+#line 508 "../../src/configparser.y"
 {
   yygotominor.yy18 = yymsp[0].minor.yy18;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1313 "./configparser.c"
+#line 1315 "./configparser.c"
         break;
       case 33:
-#line 511 "../../src/configparser.y"
+#line 513 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1327,14 +1329,14 @@ static void yy_reduce(
     yygotominor.yy18 = cur;
   }
 }
-#line 1330 "./configparser.c"
+#line 1332 "./configparser.c"
         /* No destructor defined for context */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 34:
-#line 525 "../../src/configparser.y"
+#line 527 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1348,14 +1350,14 @@ static void yy_reduce(
     yygotominor.yy18 = cur;
   }
 }
-#line 1351 "./configparser.c"
+#line 1353 "./configparser.c"
         /* No destructor defined for context_else */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 35:
-#line 539 "../../src/configparser.y"
+#line 541 "../../src/configparser.y"
 {
   data_config *dc;
   buffer *b = NULL, *rvalue, *op = NULL;
@@ -1549,13 +1551,13 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1552 "./configparser.c"
+#line 1554 "./configparser.c"
   yy_destructor(17,&yymsp[-6].minor);
   yy_destructor(19,&yymsp[-4].minor);
   yy_destructor(20,&yymsp[-2].minor);
         break;
       case 36:
-#line 733 "../../src/configparser.y"
+#line 735 "../../src/configparser.y"
 {
   if (ctx->ok) {
     data_config *dc = data_config_init();
@@ -1566,42 +1568,42 @@ static void yy_reduce(
     configparser_push(ctx, dc, 1);
   }
 }
-#line 1569 "./configparser.c"
+#line 1571 "./configparser.c"
         break;
       case 37:
-#line 744 "../../src/configparser.y"
+#line 746 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_EQ;
 }
-#line 1576 "./configparser.c"
+#line 1578 "./configparser.c"
   yy_destructor(21,&yymsp[0].minor);
         break;
       case 38:
-#line 747 "../../src/configparser.y"
+#line 749 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_MATCH;
 }
-#line 1584 "./configparser.c"
+#line 1586 "./configparser.c"
   yy_destructor(22,&yymsp[0].minor);
         break;
       case 39:
-#line 750 "../../src/configparser.y"
+#line 752 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_NE;
 }
-#line 1592 "./configparser.c"
+#line 1594 "./configparser.c"
   yy_destructor(23,&yymsp[0].minor);
         break;
       case 40:
-#line 753 "../../src/configparser.y"
+#line 755 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_NOMATCH;
 }
-#line 1600 "./configparser.c"
+#line 1602 "./configparser.c"
   yy_destructor(24,&yymsp[0].minor);
         break;
       case 41:
-#line 757 "../../src/configparser.y"
+#line 759 "../../src/configparser.y"
 {
   yygotominor.yy29 = NULL;
   if (ctx->ok) {
@@ -1619,10 +1621,10 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1622 "./configparser.c"
+#line 1624 "./configparser.c"
         break;
       case 42:
-#line 775 "../../src/configparser.y"
+#line 777 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_file(ctx->srv, ctx, yymsp[0].minor.yy29->ptr)) {
@@ -1632,11 +1634,11 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy29);
   yymsp[0].minor.yy29 = NULL;
 }
-#line 1635 "./configparser.c"
+#line 1637 "./configparser.c"
   yy_destructor(25,&yymsp[-1].minor);
         break;
       case 43:
-#line 785 "../../src/configparser.y"
+#line 787 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_cmd(ctx->srv, ctx, yymsp[0].minor.yy29->ptr)) {
@@ -1646,7 +1648,7 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy29);
   yymsp[0].minor.yy29 = NULL;
 }
-#line 1649 "./configparser.c"
+#line 1651 "./configparser.c"
   yy_destructor(26,&yymsp[-1].minor);
         break;
   };
@@ -1680,7 +1682,7 @@ static void yy_parse_failed(
 
   ctx->ok = 0;
 
-#line 1683 "./configparser.c"
+#line 1685 "./configparser.c"
   configparserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
