@@ -439,7 +439,7 @@ handler_t connection_handle_read_post_state(server *srv, connection *con) {
 	    && chunkqueue_is_empty(con->write_queue) && con->is_writable) {
 		buffer *vb = http_header_request_get(con, HTTP_HEADER_EXPECT, CONST_STR_LEN("Expect"));
 		if (NULL != vb && 0 == buffer_caseless_compare(CONST_BUF_LEN(vb), CONST_STR_LEN("100-continue"))) {
-			buffer_reset(vb); /* unset value in request headers */
+			http_header_request_unset(con, HTTP_HEADER_EXPECT, CONST_STR_LEN("Expect"));
 			if (!connection_write_100_continue(srv, con)) {
 				return HANDLER_ERROR;
 			}
@@ -494,13 +494,13 @@ void connection_response_reset(server *srv, connection *con) {
 	con->file_started = 0;
 	con->response.keep_alive = 0;
 	if (con->physical.path) { /*(skip for mod_fastcgi authorizer)*/
-		buffer_reset(con->physical.doc_root);
+		buffer_clear(con->physical.doc_root);
 		buffer_reset(con->physical.path);
-		buffer_reset(con->physical.basedir);
+		buffer_clear(con->physical.basedir);
 		buffer_reset(con->physical.rel_path);
-		buffer_reset(con->physical.etag);
+		buffer_clear(con->physical.etag);
 	}
 	con->response.htags = 0;
-	array_reset(con->response.headers);
+	array_reset_data_strings(con->response.headers);
 	http_response_body_clear(con, 0);
 }
