@@ -40,33 +40,17 @@ static connection *connections_get_new_connection(server *srv) {
 	connections *conns = srv->conns;
 	size_t i;
 
-	if (conns->size == 0) {
-		conns->size = srv->max_conns >= 128 ? 128 : srv->max_conns > 16 ? 16 : srv->max_conns;
-		conns->ptr = NULL;
-		conns->ptr = malloc(sizeof(*conns->ptr) * conns->size);
-		force_assert(NULL != conns->ptr);
-		for (i = 0; i < conns->size; i++) {
-			conns->ptr[i] = connection_init(srv);
-		}
-	} else if (conns->size == conns->used) {
-		conns->size += srv->max_conns >= 128 ? 128 : 16;
+	if (conns->size == conns->used) {
+		conns->size += srv->max_conns >= 128 ? 128 : srv->max_conns > 16 ? 16 : srv->max_conns;
 		conns->ptr = realloc(conns->ptr, sizeof(*conns->ptr) * conns->size);
 		force_assert(NULL != conns->ptr);
 
 		for (i = conns->used; i < conns->size; i++) {
 			conns->ptr[i] = connection_init(srv);
+			connection_reset(srv, conns->ptr[i]);
 		}
 	}
 
-	connection_reset(srv, conns->ptr[conns->used]);
-#if 0
-	fprintf(stderr, "%s.%d: add: ", __FILE__, __LINE__);
-	for (i = 0; i < conns->used + 1; i++) {
-		fprintf(stderr, "%d ", conns->ptr[i]->fd);
-	}
-	fprintf(stderr, "\n");
-#endif
-
 	conns->ptr[conns->used]->ndx = conns->used;
 	return conns->ptr[conns->used++];
 }
@@ -80,7 +64,7 @@ static int connection_del(server *srv, connection *con) {
 
 	if (-1 == con->ndx) return -1;
 
-	buffer_reset(con->uri.authority);
+	buffer_clear(con->uri.authority);
 	buffer_reset(con->uri.path);
 	buffer_reset(con->uri.query);
 	buffer_reset(con->request.orig_uri);
@@ -245,7 +229,7 @@ static void connection_handle_errdoc_init(connection *con) {
 
 	buffer_reset(con->physical.path);
 	con->response.htags = 0;
-	array_reset(con->response.headers);
+	array_reset_data_strings(con->response.headers);
 	http_response_body_clear(con, 0);
 
 	if (NULL != www_auth) {
@@ -326,12 +310,11 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 		}
 
 		if (!con->file_finished) {
-			buffer *b;
+			buffer *b = srv->tmp_buf;
 
 			buffer_reset(con->physical.path);
 
 			con->file_finished = 1;
-			b = buffer_init();
 
 			/* build default error-page */
 			buffer_copy_string_len(b, CONST_STR_LEN(
@@ -355,8 +338,7 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 					     "</html>\n"
 					     ));
 
-			(void)http_chunk_append_buffer(srv, con, b);
-			buffer_free(b);
+			(void)http_chunk_append_mem(srv, con, CONST_BUF_LEN(b));
 
 			http_header_response_set(con, HTTP_HEADER_CONTENT_TYPE, CONST_STR_LEN("Content-Type"), CONST_STR_LEN("text/html"));
 		}
@@ -391,8 +373,7 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 			    con->http_status == 204 ||
 			    con->http_status == 304) {
 				/* no Content-Body, no Content-Length */
-				buffer *vb = http_header_response_get(con, HTTP_HEADER_CONTENT_LENGTH, CONST_STR_LEN("Content-Length"));
-				if (NULL != vb) buffer_reset(vb); /* Headers with empty values are ignored for output */
+				http_header_response_unset(con, HTTP_HEADER_CONTENT_LENGTH, CONST_STR_LEN("Content-Length"));
 			} else if (qlen > 0 || con->request.http_method != HTTP_METHOD_HEAD) {
 				/* qlen = 0 is important for Redirects (301, ...) as they MAY have
 				 * a content. Browsers are waiting for a Content otherwise
@@ -420,11 +401,10 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 				con->response.send_chunked = 1;
 				if (qlen) {
 					/* create initial Transfer-Encoding: chunked segment */
-					buffer *b = srv->tmp_chunk_len;
-					buffer_string_set_length(b, 0);
+					buffer * const b = chunkqueue_prepend_buffer_open(con->write_queue);
 					buffer_append_uint_hex(b, (uintmax_t)qlen);
 					buffer_append_string_len(b, CONST_STR_LEN("\r\n"));
-					chunkqueue_prepend_buffer(con->write_queue, b);
+					chunkqueue_prepend_buffer_commit(con->write_queue);
 					chunkqueue_append_mem(con->write_queue, CONST_STR_LEN("\r\n"));
 				}
 				http_header_response_append(con, HTTP_HEADER_TRANSFER_ENCODING, CONST_STR_LEN("Transfer-Encoding"), CONST_STR_LEN("chunked"));
@@ -630,7 +610,7 @@ int connection_reset(server *srv, connection *con) {
 	con->request.http_version = HTTP_VERSION_UNSET;
 
 #define CLEAN(x) \
-	if (con->x) buffer_reset(con->x);
+	buffer_reset(con->x);
 
 	CLEAN(request.uri);
 	CLEAN(request.request_line);
@@ -639,34 +619,26 @@ int connection_reset(server *srv, connection *con) {
 
 	/* CLEAN(request.orig_uri); */
 
-	CLEAN(uri.scheme);
-	/* CLEAN(uri.authority); */
 	/* CLEAN(uri.path); */
 	CLEAN(uri.path_raw);
 	/* CLEAN(uri.query); */
 
 	CLEAN(parse_request);
 
-	CLEAN(server_name);
-	/*CLEAN(proto);*//* set to default in connection_accepted() */
 #undef CLEAN
 
-#define CLEAN(x) \
-	if (con->x) con->x->used = 0;
+	buffer_clear(con->uri.scheme);
+	/*buffer_clear(con->proto);*//* set to default in connection_accepted() */
+	/*buffer_clear(con->uri.authority);*/
+	buffer_clear(con->server_name);
 
-#undef CLEAN
-
-#define CLEAN(x) \
-		con->request.x = NULL;
-
-	CLEAN(http_host);
-#undef CLEAN
+	con->request.http_host = NULL;
 	con->request.content_length = 0;
 	con->request.te_chunked = 0;
 	con->request.htags = 0;
 
-	array_reset(con->request.headers);
-	array_reset(con->environment);
+	array_reset_data_strings(con->request.headers);
+	array_reset_data_strings(con->environment);
 
 	chunkqueue_reset(con->request_content_queue);
 
@@ -684,47 +656,16 @@ int connection_reset(server *srv, connection *con) {
 	return 0;
 }
 
-/**
- * handle all header and content read
- *
- * we get called by the state-engine and by the fdevent-handler
- */
-static int connection_handle_read_state(server *srv, connection *con)  {
+static void connection_read_header(server *srv, connection *con)  {
 	chunk *c, *last_chunk;
 	off_t last_offset;
 	chunkqueue *cq = con->read_queue;
-	int is_closed = 0; /* the connection got closed, if we don't have a complete header, -> error */
-	/* when in CON_STATE_READ: about to receive first byte for a request: */
-	int is_request_start = chunkqueue_is_empty(cq);
-
-	if (con->is_readable) {
-		con->read_idle_ts = srv->cur_ts;
-
-		switch(con->network_read(srv, con, con->read_queue, MAX_READ_LIMIT)) {
-		case -1:
-			connection_set_state(srv, con, CON_STATE_ERROR);
-			return -1;
-		case -2:
-			is_closed = 1;
-			break;
-		default:
-			break;
-		}
-	}
 
 	chunkqueue_remove_finished_chunks(cq);
 
 	/* we might have got several packets at once
 	 */
 
-	/* update request_start timestamp when first byte of
-	 * next request is received on a keep-alive connection */
-	if (con->request_count > 1 && is_request_start) {
-		con->request_start = srv->cur_ts;
-		if (con->conf.high_precision_timestamps)
-			log_clock_gettime_realtime(&con->request_start_hp);
-	}
-
 		/* if there is a \r\n\r\n in the chunkqueue
 		 *
 		 * scan the chunk-queue twice
@@ -817,10 +758,6 @@ found_header_end:
 			}
 
 			connection_set_state(srv, con, CON_STATE_REQUEST_END);
-		} else if (is_closed) {
-			/* the connection got closed and we didn't got enough data to leave CON_STATE_READ;
-			 * the only way is to leave here */
-			connection_set_state(srv, con, CON_STATE_ERROR);
 		}
 
 		if ((last_chunk ? buffer_string_length(con->request.request) : (size_t)chunkqueue_length(cq))
@@ -832,6 +769,57 @@ found_header_end:
 		}
 
 	chunkqueue_remove_finished_chunks(cq);
+}
+
+/**
+ * handle request header read
+ *
+ * we get called by the state-engine and by the fdevent-handler
+ */
+static int connection_handle_read_state(server *srv, connection *con)  {
+	int is_closed = 0; /* the connection got closed, if we don't have a complete header, -> error */
+
+	if (con->request_count > 1 && 0 == con->bytes_read) {
+
+		/* update request_start timestamp when first byte of
+		 * next request is received on a keep-alive connection */
+		con->request_start = srv->cur_ts;
+		if (con->conf.high_precision_timestamps)
+			log_clock_gettime_realtime(&con->request_start_hp);
+
+		if (!chunkqueue_is_empty(con->read_queue)) {
+			/*(if partially read next request and unable to read() any bytes below,
+			 * then will unnecessarily scan again here before subsequent read())*/
+			connection_read_header(srv, con);
+			if (con->state != CON_STATE_READ) {
+				con->read_idle_ts = srv->cur_ts;
+				return 0;
+			}
+		}
+	}
+
+	if (con->is_readable) {
+		con->read_idle_ts = srv->cur_ts;
+
+		switch (con->network_read(srv, con, con->read_queue, MAX_READ_LIMIT)) {
+		case -1:
+			connection_set_state(srv, con, CON_STATE_ERROR);
+			return -1;
+		case -2:
+			is_closed = 1;
+			break;
+		default:
+			break;
+		}
+	}
+
+	connection_read_header(srv, con);
+
+	if (con->state == CON_STATE_READ && is_closed) {
+		/* the connection got closed and we didn't got enough data to leave CON_STATE_READ;
+		 * the only way is to leave here */
+		connection_set_state(srv, con, CON_STATE_ERROR);
+	}
 
 	return 0;
 }
@@ -975,25 +963,23 @@ connection *connection_accept(server *srv, server_socket *srv_socket) {
 
 /* 0: everything ok, -1: error, -2: con closed */
 static int connection_read_cq(server *srv, connection *con, chunkqueue *cq, off_t max_bytes) {
-	int len;
+	ssize_t len;
 	char *mem = NULL;
 	size_t mem_len = 0;
-	int toread;
 	force_assert(cq == con->read_queue);       /*(code transform assumption; minimize diff)*/
 	force_assert(max_bytes == MAX_READ_LIMIT); /*(code transform assumption; minimize diff)*/
 
-	/* default size for chunks is 4kb; only use bigger chunks if FIONREAD tells
-	 *  us more than 4kb is available
-	 * if FIONREAD doesn't signal a big chunk we fill the previous buffer
-	 *  if it has >= 1kb free
+	/* check avail data to read and obtain memory into which to read
+	 * fill previous chunk if it has sufficient space
+	 * (use mem_len=0 to obtain large buffer at least half of chunk_buf_sz)
 	 */
-	if (0 != fdevent_ioctl_fionread(con->fd, S_IFSOCK, &toread) || toread <= 4096) {
-		toread = 4096;
+	{
+		int frd;
+		if (0 == fdevent_ioctl_fionread(con->fd, S_IFSOCK, &frd)) {
+			mem_len = (frd < MAX_READ_LIMIT) ? (size_t)frd : MAX_READ_LIMIT;
+		}
 	}
-	else if (toread > MAX_READ_LIMIT) {
-		toread = MAX_READ_LIMIT;
-	}
-	chunkqueue_get_memory(con->read_queue, &mem, &mem_len, 0, toread);
+	mem = chunkqueue_get_memory(con->read_queue, &mem_len);
 
 #if defined(__WIN32)
 	len = recv(con->fd, mem, mem_len, 0);
@@ -1100,6 +1086,7 @@ connection *connection_accepted(server *srv, server_socket *srv_socket, sock_add
 
 		buffer_copy_string_len(con->proto, CONST_STR_LEN("http"));
 		if (HANDLER_GO_ON != plugins_call_handle_connection_accept(srv, con)) {
+			connection_reset(srv, con);
 			connection_close(srv, con);
 			return NULL;
 		}
@@ -1140,7 +1127,7 @@ int connection_state_machine(server *srv, connection *con) {
 
 			break;
 		case CON_STATE_REQUEST_END: /* transient */
-			buffer_reset(con->uri.authority);
+			buffer_clear(con->uri.authority);
 			buffer_reset(con->uri.path);
 			buffer_reset(con->uri.query);
 			buffer_reset(con->request.orig_uri);
