@@ -23,13 +23,7 @@
 #include <unistd.h>
 
 
-int http_response_redirect_to_directory(server *srv, connection *con) {
-	buffer *o;
-
-	o = buffer_init();
-
-	buffer_copy_buffer(o, con->uri.scheme);
-	buffer_append_string_len(o, CONST_STR_LEN("://"));
+int http_response_buffer_append_authority(server *srv, connection *con, buffer *o) {
 	if (!buffer_string_is_empty(con->uri.authority)) {
 		buffer_append_string_buffer(o, con->uri.authority);
 	} else {
@@ -37,35 +31,73 @@ int http_response_redirect_to_directory(server *srv, connection *con) {
 		sock_addr our_addr;
 		socklen_t our_addr_len;
 
+		our_addr.plain.sa_family = 0;
 		our_addr_len = sizeof(our_addr);
 
 		if (-1 == getsockname(con->fd, (struct sockaddr *)&our_addr, &our_addr_len)
 		    || our_addr_len > (socklen_t)sizeof(our_addr)) {
 			con->http_status = 500;
-
 			log_error_write(srv, __FILE__, __LINE__, "ss",
 					"can't get sockname", strerror(errno));
-
-			buffer_free(o);
-			return 0;
+			return -1;
 		}
 
+		if (our_addr.plain.sa_family == AF_INET
+		    && our_addr.ipv4.sin_addr.s_addr == htonl(INADDR_LOOPBACK)) {
+			static char lhost[32];
+			static size_t lhost_len = 0;
+			if (0 != lhost_len) {
+				buffer_append_string_len(o, lhost, lhost_len);
+			}
+			else {
+				size_t olen = buffer_string_length(o);
+				if (0 == sock_addr_nameinfo_append_buffer(srv, o, &our_addr)) {
+					lhost_len = buffer_string_length(o) - olen;
+					if (lhost_len < sizeof(lhost)) {
+						memcpy(lhost, o->ptr+olen, lhost_len+1); /*(+1 for '\0')*/
+					}
+					else {
+						lhost_len = 0;
+					}
+				}
+				else {
+					lhost_len = sizeof("localhost")-1;
+					memcpy(lhost, "localhost", lhost_len+1); /*(+1 for '\0')*/
+					buffer_append_string_len(o, lhost, lhost_len);
+				}
+			}
+		} else if (!buffer_string_is_empty(con->server_name)) {
+			buffer_append_string_buffer(o, con->server_name);
+		} else
 		/* Lookup name: secondly try to get hostname for bind address */
 		if (0 != sock_addr_nameinfo_append_buffer(srv, o, &our_addr)) {
 			con->http_status = 500;
-			buffer_free(o);
 			return -1;
-		} else {
+		}
+
+		{
+			unsigned short listen_port = sock_addr_get_port(&our_addr);
 			unsigned short default_port = 80;
-			if (buffer_is_equal_caseless_string(con->uri.scheme, CONST_STR_LEN("https"))) {
+			if (buffer_is_equal_string(con->uri.scheme, CONST_STR_LEN("https"))) {
 				default_port = 443;
 			}
-			if (default_port != srv->srvconf.port) {
+			if (0 == listen_port) listen_port = srv->srvconf.port;
+			if (default_port != listen_port) {
 				buffer_append_string_len(o, CONST_STR_LEN(":"));
-				buffer_append_int(o, srv->srvconf.port);
+				buffer_append_int(o, listen_port);
 			}
 		}
 	}
+	return 0;
+}
+
+int http_response_redirect_to_directory(server *srv, connection *con) {
+	buffer *o = srv->tmp_buf;
+	buffer_copy_buffer(o, con->uri.scheme);
+	buffer_append_string_len(o, CONST_STR_LEN("://"));
+	if (0 != http_response_buffer_append_authority(srv, con, o)) {
+		return -1;
+	}
 	buffer_append_string_encoded(o, CONST_BUF_LEN(con->uri.path), ENCODING_REL_URI);
 	buffer_append_string_len(o, CONST_STR_LEN("/"));
 	if (!buffer_string_is_empty(con->uri.query)) {
@@ -77,9 +109,6 @@ int http_response_redirect_to_directory(server *srv, connection *con) {
 
 	con->http_status = 301;
 	con->file_finished = 1;
-
-	buffer_free(o);
-
 	return 0;
 }
 
@@ -217,7 +246,8 @@ static int http_response_parse_range(server *srv, connection *con, buffer *path,
 	for (s = range, error = 0;
 	     !error && *s && NULL != (minus = strchr(s, '-')); ) {
 		char *err;
-		off_t la, le;
+		off_t la = 0, le;
+		*((const char **)&err) = s; /*(quiet clang --analyze)*/
 
 		if (s != minus) {
 			la = strtoll(s, &err, 10);
@@ -1410,8 +1440,7 @@ int http_cgi_headers (server *srv, connection *con, http_cgi_opts *opts, http_cg
     rc |= cb(vdata, CONST_STR_LEN("REQUEST_SCHEME"),
                     CONST_BUF_LEN(con->uri.scheme));
 
-    if (buffer_is_equal_caseless_string(con->uri.scheme,
-                                        CONST_STR_LEN("https"))) {
+    if (buffer_is_equal_string(con->uri.scheme, CONST_STR_LEN("https"))) {
         rc |= cb(vdata, CONST_STR_LEN("HTTPS"), CONST_STR_LEN("on"));
     }
 
