@@ -298,7 +298,7 @@ SETDEFAULTS_FUNC(mod_deflate_setdefaults) {
 		s->compression_level = -1;
 		s->max_loadavg = 0.0;
 
-		array_reset(p->encodings); /* temp array for allowed encodings list */
+		array_reset_data_strings(p->encodings); /* temp array for allowed encodings list */
 
 		cv[0].destination = s->mimetypes;
 		cv[1].destination = p->encodings;
@@ -308,7 +308,7 @@ SETDEFAULTS_FUNC(mod_deflate_setdefaults) {
 		cv[5].destination = &(s->output_buffer_size);
 		cv[6].destination = &(s->work_block_size);
 		cv[7].destination = p->tmp_buf;
-		buffer_string_set_length(p->tmp_buf, 0);
+		buffer_clear(p->tmp_buf);
 
 		p->config_storage[i] = s;
 
@@ -732,8 +732,8 @@ static int mod_deflate_file_chunk(server *srv, connection *con, handler_ctx *hct
 #endif
 
 	if (-1 == c->file.fd) {  /* open the file if not already open */
-		if (-1 == (c->file.fd = fdevent_open_cloexec(c->file.name->ptr, O_RDONLY, 0))) {
-			log_error_write(srv, __FILE__, __LINE__, "sbs", "open failed for:", c->file.name, strerror(errno));
+		if (-1 == (c->file.fd = fdevent_open_cloexec(c->mem->ptr, O_RDONLY, 0))) {
+			log_error_write(srv, __FILE__, __LINE__, "sbs", "open failed for:", c->mem, strerror(errno));
 
 			return -1;
 		}
@@ -794,7 +794,7 @@ static int mod_deflate_file_chunk(server *srv, connection *con, handler_ctx *hct
 		if (MAP_FAILED == (c->file.mmap.start = mmap(0, (size_t)to_mmap, PROT_READ, MAP_SHARED, c->file.fd, c->file.mmap.offset))
 		    && (errno != EINVAL || MAP_FAILED == (c->file.mmap.start = mmap(0, (size_t)to_mmap, PROT_READ, MAP_PRIVATE, c->file.fd, c->file.mmap.offset)))) {
 			log_error_write(srv, __FILE__, __LINE__, "ssbd", "mmap failed:",
-					strerror(errno), c->file.name, c->file.fd);
+					strerror(errno), c->mem, c->file.fd);
 
 			return -1;
 		}
@@ -805,7 +805,7 @@ static int mod_deflate_file_chunk(server *srv, connection *con, handler_ctx *hct
 		if (c->file.mmap.length > (64 KByte) &&
 		    0 != madvise(c->file.mmap.start, c->file.mmap.length, MADV_WILLNEED)) {
 			log_error_write(srv, __FILE__, __LINE__, "ssbd", "madvise failed:",
-					strerror(errno), c->file.name, c->file.fd);
+					strerror(errno), c->mem, c->file.fd);
 		}
 #endif
 
@@ -834,7 +834,7 @@ static int mod_deflate_file_chunk(server *srv, connection *con, handler_ctx *hct
 		toSend = st_size;
 		if (toSend > 2 MByte) toSend = 2 MByte;
 		if (NULL == (start = malloc((size_t)toSend)) || -1 == lseek(c->file.fd, abs_offset, SEEK_SET) || toSend != read(c->file.fd, start, (size_t)toSend)) {
-			log_error_write(srv, __FILE__, __LINE__, "sbss", "reading", c->file.name, "failed:", strerror(errno));
+			log_error_write(srv, __FILE__, __LINE__, "sbss", "reading", c->mem, "failed:", strerror(errno));
 
 			free(start);
 			return -1;
@@ -850,7 +850,7 @@ static int mod_deflate_file_chunk(server *srv, connection *con, handler_ctx *hct
 			sigbus_jmp_valid = 0;
 
 			log_error_write(srv, __FILE__, __LINE__, "sbd", "SIGBUS in mmap:",
-				c->file.name, c->file.fd);
+					c->mem, c->file.fd);
 			return -1;
 		}
 	}
@@ -1184,7 +1184,7 @@ CONNECTION_FUNC(mod_deflate_handle_response_start) {
 	hctx->plugin_data = p;
 	hctx->compression_type = compression_type;
 	/* setup output buffer */
-	buffer_string_set_length(p->tmp_buf, 0);
+	buffer_clear(p->tmp_buf);
 	hctx->output = p->tmp_buf;
 	if (0 != mod_deflate_stream_init(hctx)) {
 		/*(should not happen unless ENOMEM)*/
@@ -1196,13 +1196,12 @@ CONNECTION_FUNC(mod_deflate_handle_response_start) {
 			vb->ptr[etaglen-1] = '"'; /*(overwrite '-')*/
 			buffer_string_set_length(vb, etaglen);
 		}
-		vb = http_header_response_get(con, HTTP_HEADER_CONTENT_ENCODING, CONST_STR_LEN("Content-Encoding"));
-		if (vb) buffer_reset(vb); /* headers with empty values are ignored for output */
+		http_header_response_unset(con, HTTP_HEADER_CONTENT_ENCODING, CONST_STR_LEN("Content-Encoding"));
 		return HANDLER_GO_ON;
 	}
 
 	if (con->response.htags & HTTP_HEADER_CONTENT_LENGTH) {
-		http_header_response_set(con, HTTP_HEADER_CONTENT_LENGTH, CONST_STR_LEN("Content-Length"), CONST_STR_LEN(""));
+		http_header_response_unset(con, HTTP_HEADER_CONTENT_LENGTH, CONST_STR_LEN("Content-Length"));
 	}
 	con->plugin_ctx[p->id] = hctx;
 
