@@ -326,7 +326,7 @@ SETDEFAULTS_FUNC(mod_dirlisting_set_defaults) {
 				buffer_copy_string_len(s->show_readme, CONST_STR_LEN("README.txt"));
 			}
 			else if (buffer_is_equal_string(s->show_readme, CONST_STR_LEN("disable"))) {
-				buffer_string_set_length(s->show_readme, 0);
+				buffer_clear(s->show_readme);
 			}
 		}
 
@@ -335,7 +335,7 @@ SETDEFAULTS_FUNC(mod_dirlisting_set_defaults) {
 				buffer_copy_string_len(s->show_header, CONST_STR_LEN("HEADER.txt"));
 			}
 			else if (buffer_is_equal_string(s->show_header, CONST_STR_LEN("disable"))) {
-				buffer_string_set_length(s->show_header, 0);
+				buffer_clear(s->show_header);
 			}
 		}
 	}
@@ -807,8 +807,7 @@ static void http_list_directory_header(server *srv, connection *con, plugin_data
 		buffer *hb = p->conf.show_header;
 		if (hb->ptr[0] != '/') {
 			buffer_copy_buffer(p->tmp_buf, con->physical.path);
-			buffer_append_slash(p->tmp_buf);
-			buffer_append_string_buffer(p->tmp_buf, p->conf.show_header);
+			buffer_append_path_len(p->tmp_buf, CONST_BUF_LEN(p->conf.show_header));
 			hb = p->tmp_buf;
 		}
 
@@ -858,8 +857,7 @@ static void http_list_directory_footer(server *srv, connection *con, plugin_data
 		buffer *rb = p->conf.show_readme;
 		if (rb->ptr[0] != '/') {
 			buffer_copy_buffer(p->tmp_buf,  con->physical.path);
-			buffer_append_slash(p->tmp_buf);
-			buffer_append_string_buffer(p->tmp_buf, p->conf.show_readme);
+			buffer_append_path_len(p->tmp_buf, CONST_BUF_LEN(p->conf.show_readme));
 			rb = p->tmp_buf;
 		}
 
@@ -1052,7 +1050,7 @@ static int http_list_directory(server *srv, connection *con, plugin_data *p, buf
 
 	if (files.used) http_dirls_sort(files.ent, files.used);
 
-	out = buffer_init();
+	out = chunkqueue_append_buffer_open(con->write_queue);
 	http_list_directory_header(srv, con, p, out);
 
 	/* directories */
@@ -1144,9 +1142,8 @@ static int http_list_directory(server *srv, connection *con, plugin_data *p, buf
 		http_header_response_set(con, HTTP_HEADER_CONTENT_TYPE, CONST_STR_LEN("Content-Type"), CONST_BUF_LEN(p->content_charset));
 	}
 
+	chunkqueue_append_buffer_commit(con->write_queue);
 	con->file_finished = 1;
-	chunkqueue_append_buffer(con->write_queue, out);
-	buffer_free(out);
 
 	return 0;
 }
