@@ -538,7 +538,7 @@ static buffer *magnet_env_get_buffer_by_id(server *srv, connection *con, int id)
 	case MAGNET_ENV_URI_QUERY: dest = con->uri.query; break;
 
 	case MAGNET_ENV_REQUEST_METHOD:
-		buffer_string_set_length(srv->tmp_buf, 0);
+		buffer_clear(srv->tmp_buf);
 		http_method_append(srv->tmp_buf, con->request.http_method);
 		dest = srv->tmp_buf;
 		break;
@@ -672,7 +672,9 @@ static int magnet_copy_response_header(connection *con, lua_State *L, int lighty
 				const_buffer val = magnet_checkconstbuffer(L, -1);
 				enum http_header_e id = http_header_hkey_get(key.ptr, key.len);
 
-				http_header_response_set(con, id, key.ptr, key.len, val.ptr, val.len);
+				val.len
+				  ? http_header_response_set(con, id, key.ptr, key.len, val.ptr, val.len)
+				  : http_header_response_unset(con, id, key.ptr, key.len);
 			}
 
 			lua_pop(L, 1);
