@@ -227,7 +227,9 @@ URIHANDLER_FUNC(mod_setenv_uri_handler) {
 	for (k = 0; k < hctx->conf.set_request_header->used; ++k) {
 		data_string *ds = (data_string *)hctx->conf.set_request_header->data[k];
 		enum http_header_e id = http_header_hkey_get(CONST_BUF_LEN(ds->key));
-		http_header_request_set(con, id, CONST_BUF_LEN(ds->key), CONST_BUF_LEN(ds->value));
+		!buffer_string_is_empty(ds->value)
+		  ? http_header_request_set(con, id, CONST_BUF_LEN(ds->key), CONST_BUF_LEN(ds->value))
+		  : http_header_request_unset(con, id, CONST_BUF_LEN(ds->key));
 	}
 
 	return HANDLER_GO_ON;
@@ -269,7 +271,9 @@ CONNECTION_FUNC(mod_setenv_handle_response_start) {
 	for (size_t k = 0; k < hctx->conf.set_response_header->used; ++k) {
 		data_string *ds = (data_string *)hctx->conf.set_response_header->data[k];
 		enum http_header_e id = http_header_hkey_get(CONST_BUF_LEN(ds->key));
-		http_header_response_set(con, id, CONST_BUF_LEN(ds->key), CONST_BUF_LEN(ds->value));
+		!buffer_string_is_empty(ds->value)
+		  ? http_header_response_set(con, id, CONST_BUF_LEN(ds->key), CONST_BUF_LEN(ds->value))
+		  : http_header_response_unset(con, id, CONST_BUF_LEN(ds->key));
 	}
 
 	return HANDLER_GO_ON;
