@@ -2,7 +2,7 @@
 #define _MOD_SSI_H_
 #include "first.h"
 
-#include "base.h"
+#include "base_decls.h"
 #include "buffer.h"
 #include "array.h"
 
