@@ -223,7 +223,7 @@ SETDEFAULTS_FUNC(mod_vhostdb_set_defaults) {
         { NULL,             NULL, T_CONFIG_UNSET,  T_CONFIG_SCOPE_UNSET }
     };
 
-    p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
+    p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
 
     for (size_t i = 0; i < srv->config_context->used; ++i) {
         data_config const *config = (data_config const*)srv->config_context->data[i];
