@@ -214,7 +214,7 @@ SETDEFAULTS_FUNC(mod_wstunnel_set_defaults) {
         { NULL,                     NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
     };
 
-    p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
+    p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
     force_assert(p->config_storage);
     for (size_t i = 0; i < srv->config_context->used; ++i) {
         array *ca = ((data_config *)(srv->config_context->data[i]))->value;
