@@ -253,16 +253,24 @@ int http_request_host_normalize(buffer *b, int scheme_port) {
             buffer_string_set_length(b, (size_t)(colon - p)); /*(remove port str)*/
         }
 
-        if (light_isdigit(*p)) {
+        if (light_isdigit(*p)) do {
             /* (IPv4 address literal or domain starting w/ digit (e.g. 3com))*/
+            /* (check one-element cache of normalized IPv4 address string) */
+            static struct { char s[INET_ADDRSTRLEN]; size_t n; } laddr;
+            size_t n = colon ? (size_t)(colon - p) : blen;
             sock_addr addr;
+            if (n == laddr.n && 0 == memcmp(p, laddr.s, n)) break;
             if (1 == sock_addr_inet_pton(&addr, p, AF_INET, 0)) {
                 sock_addr_inet_ntop_copy_buffer(b, &addr);
+                n = buffer_string_length(b);
+                if (n < sizeof(laddr.s)) memcpy(laddr.s, b->ptr, (laddr.n = n));
             }
-        }
-    } else { /* IPv6 addr */
+        } while (0);
+    } else do { /* IPv6 addr */
       #if defined(HAVE_IPV6) && defined(HAVE_INET_PTON)
 
+        /* (check one-element cache of normalized IPv4 address string) */
+        static struct { char s[INET6_ADDRSTRLEN]; size_t n; } laddr;
         sock_addr addr;
         char *bracket = b->ptr+blen-1;
         char *percent = strchr(b->ptr+1, '%');
@@ -286,6 +294,13 @@ int http_request_host_normalize(buffer *b, int scheme_port) {
             }
         }
 
+        len = (size_t)((percent ? percent : bracket) - (b->ptr+1));
+        if (laddr.n == len && 0 == memcmp(laddr.s, b->ptr+1, len)) {
+            /* truncate after ']' and re-add normalized port, if needed */
+            buffer_string_set_length(b, (size_t)(bracket - b->ptr + 1));
+            break;
+        }
+
         *bracket = '\0';/*(terminate IPv6 string)*/
         if (percent) *percent = '\0'; /*(remove %interface from address)*/
         rc = sock_addr_inet_pton(&addr, b->ptr+1, AF_INET6, 0);
@@ -298,6 +313,7 @@ int http_request_host_normalize(buffer *b, int scheme_port) {
         if (percent) {
             if (percent > bracket) return -1;
             if (len + (size_t)(bracket - percent) >= sizeof(buf)) return -1;
+            if (len < sizeof(laddr.s)) memcpy(laddr.s, buf, (laddr.n = len));
             memcpy(buf+len, percent, (size_t)(bracket - percent));
             len += (size_t)(bracket - percent);
         }
@@ -310,7 +326,7 @@ int http_request_host_normalize(buffer *b, int scheme_port) {
         return -1;
 
       #endif
-    }
+    } while (0);
 
     if (0 != port && port != scheme_port) {
         buffer_append_string_len(b, CONST_STR_LEN(":"));
@@ -332,10 +348,6 @@ int http_request_host_policy (connection *con, buffer *b, const buffer *scheme)
                 && 0 != http_request_host_normalize(b, scheme_port(scheme))));
 }
 
-#if 0
-#define DUMP_HEADER
-#endif
-
 static int http_request_split_value(array *vals, const char *current, size_t len) {
 	int state = 0;
 	const char *token_start = NULL, *token_end = NULL;
@@ -452,6 +464,14 @@ static int parse_single_header(server *srv, connection *con, parse_header_state
       case HTTP_HEADER_HOST:
         if (!(con->request.htags & HTTP_HEADER_HOST)) {
             saveb = &con->request.http_host;
+            if (vlen >= 1024) { /*(expecting < 256)*/
+                if (srv->srvconf.log_request_header_on_error) {
+                    log_error_write(srv, __FILE__, __LINE__, "s", "uri-authority too long -> 400");
+                    log_error_write(srv, __FILE__, __LINE__, "Sb",
+                                    "request-header:\n", con->request.request);
+                }
+                return 0; /* invalid header */
+            }
         }
         else if (state->reqline_host) {
             /* ignore all Host: headers as we got Host in request line */
@@ -470,7 +490,7 @@ static int parse_single_header(server *srv, connection *con, parse_header_state
       case HTTP_HEADER_CONNECTION:
         {
             array * const vals = srv->split_vals;
-            array_reset(vals);
+            array_reset_data_strings(vals);
             http_request_split_value(vals, v, vlen); /* split on , */
             for (size_t vi = 0; vi < vals->used; ++vi) {
                 data_string *dsv = (data_string *)vals->data[vi];
@@ -851,6 +871,14 @@ static size_t http_request_parse_reqline(server *srv, connection *con, parse_hea
 
 	if (state->reqline_host) {
 		/* Insert as host header */
+		if (state->reqline_hostlen >= 1024) { /*(expecting < 256)*/
+			if (srv->srvconf.log_request_header_on_error) {
+				log_error_write(srv, __FILE__, __LINE__, "s", "uri-authority too long -> 400");
+				log_error_write(srv, __FILE__, __LINE__, "Sb",
+						"request-header:\n", con->request.request);
+			}
+			return 0;
+		}
 		http_header_request_set(con, HTTP_HEADER_HOST, CONST_STR_LEN("Host"), state->reqline_host, state->reqline_hostlen);
 		con->request.http_host = http_header_request_get(con, HTTP_HEADER_HOST, CONST_STR_LEN("Host"));
 	}
@@ -1111,12 +1139,11 @@ int http_request_parse(server *srv, connection *con) {
 
 			/* reset value for Transfer-Encoding, a hop-by-hop header,
 			 * which must not be blindly forwarded to backends */
-			buffer_reset(vb); /* headers with empty values are ignored */
+			http_header_request_unset(con, HTTP_HEADER_TRANSFER_ENCODING, CONST_STR_LEN("Transfer-Encoding"));
 
 			/*(note: ignore whether or not Content-Length was provided)*/
 		        if (con->request.htags & HTTP_HEADER_CONTENT_LENGTH) {
-				vb = http_header_request_get(con, HTTP_HEADER_CONTENT_LENGTH, CONST_STR_LEN("Content-Length"));
-				if (NULL != vb) buffer_reset(vb); /* headers with empty values are ignored */
+				http_header_request_unset(con, HTTP_HEADER_CONTENT_LENGTH, CONST_STR_LEN("Content-Length"));
 			}
 
 			state.con_length_set = 1;
